@@ -1,7 +1,7 @@
 ;;;
 ;;; configure.scm - configuring Gauche extensions
 ;;;
-;;;   Copyright (c) 2013-2016  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2013-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -48,22 +48,28 @@
 ;; to the source tree, and anyone who checks it out can directly run
 ;; ./configure, without worrying running autoconf (and free from headache
 ;; of autoconf version mismatch)
-
+;;
 ;; The core feature of gauche.configure is the ability to generate files
 ;; (e.g. Makefile) from templates (e.g. Makefile.in) with replacing
 ;; parameters.  We follow autoconf convention, so the replacement variables
 ;; in a template is written like @VAR@.
-
+;;
 ;; The API is roughly corresponds to autoconf's AC_* macros, while we use
 ;; 'cf-' suffix instead.
+;;
+;; The simplest configure scripts can be just the following 3 expressions:
+;;
+;;  (use gauche.configure)
+;;  (cf-init-gauche-extension)
+;;  (cf-output-default)
+;;
+;; It takes package name and version from package.scm file, sets several
+;; substitution variables, and creates Makefile from Makefile.in along
+;; the gpd (Gauche package description) file.
 
-;; NB: cf-define currently only suppors substituting DEFS; it doesn't
-;; handle config.h.
-
-;; TODO: Caching of test results
+;; TODO: Caching test results
 
 (define-module gauche.configure
-  (use gauche.parameter)
   (use gauche.generator)
   (use gauche.dictionary)
   (use gauche.parseopt)
@@ -81,22 +87,31 @@
   (use srfi-13)
   (use srfi-113) ; sets & bags
   (extend gauche.config)
-  (export cf-init
+  (export cf-init cf-init-gauche-extension
           cf-arg-enable cf-arg-with cf-feature-ref cf-package-ref
           cf-help-string
-          cf-msg-checking cf-msg-result cf-msg-warn cf-msg-error
+          cf-msg-checking cf-msg-result cf-msg-warn cf-msg-error cf-msg-notice
           cf-echo
           cf-make-gpd
-          cf-define cf-subst cf-arg-var cf-have-subst? cf-ref cf$
-          cf-config-headers cf-output cf-show-substs
+          cf-define cf-defined? cf-subst cf-subst-append cf-subst-prepend
+          cf-arg-var cf-have-subst? cf-ref cf$
+          with-cf-subst
+          cf-config-headers cf-output cf-output-default cf-show-substs
           cf-check-prog cf-path-prog cf-check-tool
           cf-prog-cxx
 
-          cf-check-header cf-check-headers cf-includes-default
-          
           cf-lang <c-language>
           cf-lang-program cf-lang-io-program cf-lang-call
           cf-try-compile cf-try-compile-and-link
+
+          cf-check-header cf-header-available? cf-check-headers
+          cf-includes-default
+          cf-check-type cf-type-available? cf-check-types
+          cf-check-decl cf-decl-available? cf-check-decls
+          cf-check-member cf-member-available? cf-check-members
+          
+          cf-check-func cf-func-available? cf-check-funcs
+          cf-check-lib cf-lib-available? cf-search-libs
           ))
 (select-module gauche.configure)
 
@@ -163,6 +178,8 @@
 (define (cf-msg-error fmt . args)
   (tee-msg #"Error: ~|fmt|\n" #"Error: ~fmt" args)
   (exit 1))
+(define (cf-msg-notice fmt . args)
+  (tee-msg #"~|fmt|\n" #"~|fmt|\n" args))
 
 ;; API
 ;; Convenience routine for substitute of shell's echo
@@ -188,6 +205,8 @@
 ;; has "package.scm".
 (define (cf-init :optional (package-name #f) (version #f)
                            (bug-report #f) (url #f))
+  (when (current-package)
+    (exit 1 "Incorrect configure script: cf-init is called more than once."))
   (let* ([gpd (and-let* ([srcdir (current-load-path)]
                          [pfile (build-path (sys-dirname srcdir) "package.scm")]
                          [ (file-exists? pfile) ])
@@ -244,8 +263,8 @@
                     x))
                 url)]
          )
-    (check-arg string? package-name)
-    (check-arg string? version)
+    (assume-type package-name <string>)
+    (assume-type version <string>)
     (sys-unlink "config.log")
     (log-open "config.log" :prefix "")
     (log-format "Configuring ~a ~a" package-name version)
@@ -270,6 +289,7 @@
 
 (define (initialize-default-definitions)
   (define p (current-package))
+  (define (S s) (and s (write-to-string s)))
   (cf-subst 'PACKAGE_NAME    (~ p'name))
   (cf-subst 'PACKAGE_TARNAME (string-tr (string-downcase (~ p'name))
                                         "a-z0-9_-" "_*" :complement #t))
@@ -277,6 +297,13 @@
   (cf-subst 'PACKAGE_STRING (~ p'string))
   (cf-subst 'PACKAGE_BUGREPORT (~ p'bug-report))
   (cf-subst 'PACKAGE_URL (~ p'url))
+  (cf-define 'PACKAGE_NAME    (S (~ p'name)))
+  (cf-define 'PACKAGE_TARNAME (S (string-tr (string-downcase (~ p'name))
+                                            "a-z0-9_-" "_*" :complement #t)))
+  (cf-define 'PACKAGE_VERSION (S (~ p'version)))
+  (cf-define 'PACKAGE_STRING (S (~ p'string)))
+  (cf-define 'PACKAGE_BUGREPORT (S (~ p'bug-report)))
+  (cf-define 'PACKAGE_URL (S (~ p'url)))
 
   (cf-subst 'SHELL (or (sys-getenv "CONFIG_SHELL") "/bin/sh"))
   (cf-subst 'LIBOBJS "")
@@ -309,30 +336,42 @@
   (cf-subst 'cross_compiling "no")
   (cf-subst 'subdirs "")
 
+  ;; For cross compilation.  These can be overridden by --build, --host and
+  ;; --target.
+  (cf-subst 'build  (gauche-architecture))
+  (cf-subst 'host   (gauche-architecture))
+  (cf-subst 'target (gauche-architecture))
+
   ;; NB: Autoconf uses AC_PROG_CC to set up CC.  However, we need
   ;; to use the same C compiler with which Gauche was compiled, so
-  ;; so we set it as the default.  We also allow env overrides for
-  ;; some common variables.  (In autoconf, AC_PROG_CC issues AC_ARG_VAR
+  ;; we set it as the default.  We also allow env overrides for some
+  ;; common variables.  (In autoconf, AC_PROG_CC issues AC_ARG_VAR
   ;; for them).
-  (cf-subst 'CC       (gauche-config "--cc"))
+  (cf-subst 'CC (gauche-config "--cc"))
+  (cf-subst 'CFLAGS (gauche-config "--default-cflags"))
   (cf-arg-var 'CPP)
-  (cf-arg-var 'CPPFALGS)
+  (cf-arg-var 'CPPFLAGS)
   (cf-arg-var 'CC)
   (cf-arg-var 'CFLAGS)
   (cf-arg-var 'LDFLAGS)
   (cf-arg-var 'LIBS)
-  ;; NB: Autoconf detemines these through tests, but we already
+  ;; NB: Autoconf determines these through tests, but we already
   ;; know them at the time Gauche is configured.
   (cf-subst 'SOEXT  (gauche-config "--so-suffix"))
   (cf-subst 'OBJEXT (gauche-config "--object-suffix"))
   (cf-subst 'EXEEXT (gauche-config "--executable-suffix"))
+
+  ;; '-DUNICODE' if gauche is in utf8 and on windows.
+  (cf-subst 'WINDOWS_UNICODE_FLAG
+            (cond-expand [(and gauche.os.windows gauche.ces.utf8) "-DUNICODE"]
+                         [else ""]))
   )
 
 (define (parse-command-line-arguments)
   (let1 rest
       (parse-options (cdr (command-line))
         (["bindir=s" (dir) (cf-subst 'bindir dir)]
-         ["build=s" (build) (cf-subst 'build-alias build)]
+         ["build=s" (build) (cf-subst 'build_alias build)]
          ["c|cache-file=s" (_)
           (values)] ;; support for the compatibility.  we don't do anything.
          ["C|config-cache" (_) (values)]
@@ -343,7 +382,7 @@
          ;; --disable-option-checking
          ["exec_prefix=s" (pre) (cf-subst 'exec_prefix pre)]
          ["help" () (usage)]
-         ["host=s" (host) (cf-subst 'host-alias host)]
+         ["host=s" (host) (cf-subst 'host_alias host)]
          ["htmldir=s" (dir) (cf-subst 'htmldir dir)]
          ["includedir=s" (dir) (cf-subst 'includedir dir)]
          ["infodir=s" (dir) (cf-subst 'infodir dir)]
@@ -413,6 +452,10 @@
         [else (print "Invalid argument: " arg)
               (print "Type `./configure --help' for usage.")
               (exit 1)]))
+    ;; build, host and target override
+    (when (cf-have-subst? 'build_alias)  (cf-subst 'build (cf$ 'build_alias)))
+    (when (cf-have-subst? 'host_alias)   (cf-subst 'host (cf$ 'host_alias)))
+    (when (cf-have-subst? 'target_alias) (cf-subst 'target (cf$ 'target_alias)))
     ))
 
 (define (check-directory-names)
@@ -439,6 +482,7 @@
   (cf-subst 'top_builddir (cf$ 'builddir))
   )
 
+;; Process the arg-if-given and arg-if-not-given callbacks of cf-arg-*.
 (define (process-args)
   (dolist [entry (arg-processors)]
     (match entry
@@ -552,22 +596,75 @@
   ;; TODO: explanation of VAR=VAL
   (exit 1))
 
+;; API
+;; cf-init-gauche-extension packages common stuff around cf-init.
+;;
+(define (cf-init-gauche-extension)
+  (cf-arg-with 'local
+               (cf-help-string
+                "--with-local=PATH:PATH..."
+                "For each PATH, add PATH/include to the include search
+  paths and PATH/lib to the library search paths.  Useful if you have some
+  libraries installed in non-standard places. ")
+               (^[with-local]
+                 (unless (member with-local '("yes" "no" ""))
+                   (cf-subst 'LOCAL_PATHS with-local)))
+               (^[] (cf-subst 'LOCAL_PATHS "")))
+  (cf-init)
+  (cf-path-prog 'GOSH            "gosh")
+  (cf-path-prog 'GAUCHE_CONFIG   "gauche-config")
+  (cf-path-prog 'GAUCHE_PACKAGE  "gauche-package")
+  (cf-path-prog 'GAUCHE_INSTALL  "gauche-install")
+  (cf-path-prog 'GAUCHE_CESCONV  "gauche-cesconv")
+
+  (cf-subst 'default_prefix (gauche-config "--prefix"))
+ 
+  (cf-subst 'GAUCHE_PKGINCDIR  (gauche-config "--pkgincdir"))
+  (cf-subst 'GAUCHE_PKGLIBDIR  (gauche-config "--pkglibdir"))
+  (cf-subst 'GAUCHE_PKGARCHDIR (gauche-config "--pkgarchdir"))
+  )
+
 ;;;
 ;;; Variables and substitutions
 ;;;
 
 ;; API
 (define (cf-define symbol :optional (value 1))
+  (assume-type symbol <symbol>)
   (dict-put! (~ (ensure-package)'defs) symbol value))
 
+;; API
+(define (cf-defined? symbol)
+  (assume-type symbol <symbol>)
+  (dict-exists? (~ (ensure-package)'defs) symbol))
+
 ;; API
 ;; Like AC_SUBST, but we require value (instead of implicitly referencing
 ;; a global variable.
-(define (cf-subst symbol value)
-  (dict-put! (~ (ensure-package)'substs) symbol value))
+(define (cf-subst symbol :optional (value #f))
+  (assume-type symbol <symbol>)
+  (if value
+    (dict-put! (~ (ensure-package)'substs) symbol value)
+    (unless (cf-have-subst? symbol)
+      (dict-put! (~ (ensure-package)'substs) symbol ""))))
+
+;; API
+(define (cf-subst-prepend symbol value :optional (delim " ") (default ""))
+  (let1 v (cf-ref symbol default)
+    (if (equal? v "")
+      (cf-subst symbol value)
+      (cf-subst symbol #"~|value|~|delim|~|v|"))))
+
+;; API
+(define (cf-subst-append symbol value :optional (delim " ") (default ""))
+  (let1 v (cf-ref symbol default)
+    (if (equal? v "")
+      (cf-subst symbol value)
+      (cf-subst symbol #"~|v|~|delim|~|value|"))))
 
 ;; API
 (define (cf-have-subst? symbol)
+  (assume-type symbol <symbol>)
   (dict-exists? (~ (ensure-package)'substs) symbol))
 
 ;; API
@@ -577,16 +674,20 @@
 ;; called after cf-init and we can't include help message (the limitation
 ;; of one-pass processing.)
 (define (cf-arg-var symbol)
+  (assume-type symbol <symbol>)
   (update! (~ (ensure-package)'precious) (cut set-adjoin! <> symbol))
-  (and-let1 v (sys-getenv (x->string symbol))
-    (cf-subst symbol v)))
+  (if-let1 v (sys-getenv (x->string symbol))
+    (cf-subst symbol v)
+    (cf-subst symbol)))
 
 (define (var-precious? symbol)
+  (assume-type symbol <symbol>)
   (set-contains? (~ (ensure-package)'precious) symbol))
 
 ;; API
 ;; Lookup the current value of the given variable.
 (define (cf-ref symbol :optional (default (undefined)))
+  (assume-type symbol <symbol>)
   (rlet1 v (dict-get (~ (ensure-package)'substs) symbol default)
     (when (undefined? v)
       (errorf "Configure variable ~s is not defined." symbol))))
@@ -595,6 +696,20 @@
 ;; Like cf-ref, but returns empty string if undefined.
 (define (cf$ symbol) (cf-ref symbol ""))
 
+;; API
+;; Temporarily replace cf subst value
+(define-syntax with-cf-subst 
+  (syntax-rules ()
+    [(_ ((var val) ...) body ...)
+     (let ([saves (map (cut cf$ <>) '(var ...))])
+       (for-each (cut cf-subst <> <>)
+                 '(var ...)
+                 (list val ...))
+       (unwind-protect (begin body ...)
+         (for-each (cut cf-subst <> <>)
+                   '(var ...)
+                   saves)))]))
+
 ;;;
 ;;; Argument processing
 ;;;
@@ -763,6 +878,7 @@
         (error "Cannot read input file ~s" inf))
       (unless (file-is-directory? (sys-dirname f))
         (make-directory* (sys-dirname f)))
+      (cf-msg-notice "configure: creating ~a" f)
       (file-filter-for-each (make-replace-1 f) :input inf :output f
                             :temporary-file #t :leave-unchanged #t)))
   (dolist [h (~ pa'config.h)]
@@ -771,6 +887,7 @@
         (error "Cannot read input file ~s" inf))
       (unless (file-is-directory? (sys-dirname (car h)))
         (make-directory* (sys-dirname (car h))))
+      (cf-msg-notice "configure: creating ~a" (car h))
       (file-filter-for-each (make-config.h) :input inf :output (car h)
                             :temporary-file #t :leave-unchanged #t)))
   )
@@ -796,12 +913,25 @@
     (with-output-to-file gpd-file
       (cut write-gauche-package-description gpd))))
 
+;; API
+;; Packages common output
+(define (cf-output-default . output-files)
+  (cf-make-gpd)
+  (cf-echo (cf$ 'PACKAGE_VERSION) > "VERSION")
+  (let* ([pfx (cf$'srcdir)]
+         [outfiles (append
+                    ($ map (^f (string-drop (string-drop-right f 3)
+                                            (+ (string-length pfx) 1)))
+                       $ glob #"~|pfx|/**/Makefile.in")
+                    output-files)])
+    (apply cf-output outfiles)))
+
 ;;;
 ;;; Target languages
 ;;;
 
 ;; Various language-specific operations are defined as methods on
-;; the language signleton object inheriting <cf-language>.
+;; the language singleton object inheriting <cf-language>.
 ;; Methods a named with -m suffix to distinguish from cf- API, which
 ;; is a usual procedure dispatches according to the current
 ;; language.
@@ -810,7 +940,7 @@
   :metaclass <singleton-meta>)
 
 ;; API
-;; Parameter cf-lang holds a singleton instance of the current langauge.
+;; Parameter cf-lang holds a singleton instance of the current language.
 (define cf-lang (make-parameter (make <cf-language>)))
 
 (define (cf-lang-ext) (cf-lang-ext-m (cf-lang)))
@@ -896,6 +1026,7 @@
   (define (clean)
     (remove-files (glob "conftest.err*")
                   #"conftest.~(cf-lang-ext)"
+                  #"conftest.~(cf$'OBJEXT)"
                   #"conftest~(cf$'EXEEXT)"))
   (define cmd
     (if (string? command)
@@ -971,7 +1102,7 @@
 ;; API
 ;; Common feature for AC_CHECK_PROG, AC_PATH_PROG etc.
 ;; Search one of programs listed in PROGS within PATHS.
-;; PATHS can be #f - then we use PATH enviornment variable.
+;; PATHS can be #f - then we use PATH environment variable.
 ;; For each found program, FILTER is called with full path of the program;
 ;; it may return #f if the caller wants to exclude the program for some reason.
 ;; If a program found, and FILTER returns true, then this procedure returns
@@ -1088,7 +1219,7 @@
 ;; Feature Test API
 ;; Like AC_CHECK_HEADER.
 ;; Returns #t on success, #f on failure.
-(define (cf-check-header header-file :key (includes #f))
+(define (cf-header-available? header-file :key (includes #f))
   (let1 includes (or includes (cf-includes-default))
     (cf-msg-checking "~a usability" header-file)
     (rlet1 result (cf-try-compile (list includes
@@ -1096,6 +1227,7 @@
                                         #"#include <~|header-file|>\n")
                                   "")
       (cf-msg-result (if result "yes" "no")))))
+(define cf-check-header cf-header-available?) ;; autoconf compatible name
 
 ;; Feature Test API
 ;; Like AC_CHECK_HEADERS.  Besides the check, it defines HAVE_<header-file>
@@ -1104,6 +1236,186 @@
                           :key (includes #f) (if-found #f) (if-not-found #f))
   (dolist [h header-files]
     (if (cf-check-header h :includes includes)
-      (begin (cf-define #"HAVE_~(safe-variable-name h)")
+      (begin (cf-define (string->symbol #"HAVE_~(safe-variable-name h)"))
              (when if-found (if-found h)))
       (when if-not-found (if-not-found h)))))
+
+;; Feature Test API
+;; Like AC_CHECK_TYPE.
+;; Returns #t on success, #f on failure.
+;; If TYPE is a valid type, sizeof(TYPE) compiles and sizeof((TYPE)) fails.
+;; The second test is needed in case TYPE happens to be a variable.
+(define (cf-type-available? type :key (includes #f))
+  (let1 includes (or includes (cf-includes-default))
+    (cf-msg-checking "for ~a" type)
+    (rlet1 result
+        (and (cf-try-compile (list includes)
+                             #"if (sizeof (~|type|)) return 0;")
+             (not (cf-try-compile (list includes)
+                                  #"if (sizeof ((~|type|))) return 0;")))
+      (cf-msg-result (if result "yes" "no")))))
+(define cf-check-type cf-type-available?)  ; autoconf-compatible name
+
+;; Feature Test API
+;; Like AC_CHECK_TYPES.
+;; For each type in types, run cf-check-type and define HAVE_type if found.
+(define (cf-check-types types :key (includes #f)
+                                   (if-found identity)
+                                   (if-not-found identity))
+  (dolist [type types]
+    (if (cf-check-type type :includes includes)
+      (begin (cf-define (string->symbol #"HAVE_~(safe-variable-name type)"))
+             (if-found type))
+      (if-not-found type))))
+
+;; Feature Test API
+;; Like AC_CHECK_DECL
+;; Returns #t on success, #f on failure.
+;; Check SYMBOL is declared as a macro, a constant, a variable or a function.
+(define (cf-decl-available? symbol :key (includes #f))
+  (let1 includes (or includes (cf-includes-default))
+    (cf-msg-checking "whether ~a is declared" symbol)
+    (rlet1 result
+        (cf-try-compile (list includes)
+                        (list #"#ifndef ~|symbol|\n"
+                              #" (void)~|symbol|;\n"
+                              #"#endif\n"
+                              "return 0;"))
+      (cf-msg-result (if result "yes" "no")))))
+(define cf-check-decl cf-decl-available?)  ;autoconf-compatible name
+
+;; Feature Test API
+;; Like AC_CHECK_DECLS
+;; For each symbol in symbols, run cf-check-decl and define HAVE_DECL_symbol
+;; to 1 (found) or 0 (not found).
+(define (cf-check-decls symbols :key (includes #f)
+                                     (if-found identity)
+                                     (if-not-found identity))
+  (dolist [symbol symbols]
+    (let1 nam (string->symbol #"HAVE_DECL_~(safe-variable-name symbol)")
+      (if (cf-check-decl symbol :includes includes)
+        (begin (cf-define nam 1)
+               (if-found symbol))
+        (begin (cf-define nam 0)
+               (if-not-found symbol))))))
+
+;; Feature Test API
+;; Like AC_CHECK_MEMBER
+;; Works as a predicate
+(define (cf-member-available? aggregate.member :key (includes #f))
+  (receive (aggr memb) (string-scan aggregate.member #\. 'both)
+    (unless (and aggr memb)
+      (error "cf-check-member: argument doesn't contain a dot:"
+             aggregate.member))
+    (cf-msg-checking "`~a' is a member of `~a'" memb aggr)
+    (let1 includes (or includes (cf-includes-default))
+      (rlet1 result
+          (or (cf-try-compile (list includes)
+                              (list #"static ~aggr ac_aggr;\n"
+                                    #"if (ac_aggr.~|memb|) return 0;"))
+              (cf-try-compile (list includes)
+                              (list #"static ~aggr ac_aggr;\n"
+                                    #"if (sizeof ac_aggr.~|memb|) return 0;")))
+        (cf-msg-result (if result "yes" "no"))))))
+(define cf-check-member cf-member-available?) ;autoconf-compatible name
+
+;; Feature Test API
+;; Like AC_CHECK_MEMBERS
+(define (cf-check-members members :key (includes #f)
+                                       (if-found identity)
+                                       (if-not-found identity))
+  (dolist [mem members]
+    (if (cf-check-member mem :includes includes)
+      (begin (cf-define (string->symbol #"HAVE_~(safe-variable-name mem)"))
+             (if-found mem))
+      (if-not-found mem))))
+
+;; Feature Test API
+;; Like AC_CHECK_FUNC
+;; NB: autoconf has language-dependent methods (AC_LANG_FUNC_LINK_TRY)
+;; For now, we hardcode C.
+(define (cf-func-available? func)
+  (let1 includes (cf-includes-default)
+    (cf-msg-checking #"for ~func")
+    (rlet1 result ($ cf-try-compile-and-link
+                     `(,#"#define ~func innocuous_~func\n"
+                       "#ifdef __STDC__\n"
+                       "# include <limits.h>\n"
+                       "#else\n"
+                       "# include <assert.h>\n"
+                       "#endif\n"
+                       ,#"#undef ~func\n"
+                       "#ifdef __cplusplus\n"
+                       "extern \"C\"\n"
+                       "#endif\n"
+                       ,#"char ~func ();\n")
+                     `(,#"return ~func ();"))
+      (cf-msg-result (if result "yes" "no")))))
+(define cf-check-func cf-func-available?)  ;autoconf-compatible name
+
+;; Feature Test API
+;; Like AC_CHECK_FUNCS
+(define (cf-check-funcs funcs :key (if-found identity) 
+                                   (if-not-found identity))
+  (dolist [f funcs]
+    (if (cf-check-func f)
+      (begin (cf-define (string->symbol #"HAVE_~(safe-variable-name f)"))
+             (if-found f))
+      (if-not-found f))))
+
+
+(define (default-lib-found libname)
+  (when libname
+    (cf-subst-prepend 'LIBS #"-l~|libname|")
+    (cf-define (string->symbol #"HAVE_LIB~(safe-variable-name libname)")))
+  #t)
+
+(define (default-lib-not-found libname) #f)
+
+;; Feature Test API
+;; Like AC_CHECK_LIB
+(define (cf-lib-available? lib fn
+                        :key (other-libs '())
+                        (if-found default-lib-found)
+                        (if-not-found default-lib-not-found))
+  (let1 includes (cf-includes-default)
+    (cf-msg-checking "for ~a in -l~a" fn lib)
+    (if (with-cf-subst
+         ([LIBS #"-l~|lib| ~(string-join other-libs \" \") ~(cf$'LIBS)"])
+         (cf-try-compile-and-link includes
+                                  (format "extern void ~a(); ~a();" fn fn)))
+      (begin
+        (cf-msg-result "yes")
+        (if-found lib))
+      (begin
+        (cf-msg-result "no")
+        (if-not-found lib)))))
+(define cf-check-lib cf-lib-available?)    ;autoconf-compatible name
+
+(define (default-lib-search-found libname)
+  (when libname
+    (cf-subst-prepend 'LIBS #"-l~|libname|"))
+  #t)
+   
+;; Feature test API
+;; Like AC_CHECK_LIBS
+(define (cf-search-libs fn libs
+                        :key (other-libs '())
+                             (if-found default-lib-search-found) 
+                             (if-not-found default-lib-not-found))
+  (let ([includes (cf-includes-default)]
+        [xlibs #"~(string-join other-libs \" \") ~(cf$'LIBS)"])
+    (define (try lib)
+      (with-cf-subst 
+       ([LIBS (if (eq? lib 'none) xlibs #"-l~|lib| ~xlibs")])
+       (cf-try-compile-and-link includes
+                                (format "extern void ~a(); ~a();"
+                                        fn fn))))
+    (cf-msg-checking "for ~a" fn)
+    (if-let1 lib (find try (cons 'none libs))
+      (begin
+        (cf-msg-result (if (eq? lib 'none) "found" #"found in -l~|lib|"))
+        (if-found (if (eq? lib 'none) #f lib)))
+      (begin
+        (cf-msg-result "no")
+        (if-not-found #f)))))
